﻿namespace UltraGridRetainScrollPosition
{
    using System;
    using System.Collections.Generic;
    using System.Data;
    using System.Windows.Forms;

    using Infragistics.Win.UltraWinGrid;

    public partial class Form1 : Form
    {
        private ICollection<UltraGridRow> expandedRows = new HashSet<UltraGridRow>();
        private UltraGridRow firstExpandedRow;

        public Form1()
        {
            this.InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraGrid1.SetDataBinding(this.GetData(), "table1");
            this.ultraGrid1.DisplayLayout.Bands[0].SortedColumns.Add(this.ultraGrid1.DisplayLayout.Bands[0].Columns["Quantity"], true, true);
            this.ultraGrid1.DisplayLayout.ScrollBounds = ScrollBounds.ScrollToLastItem;
            
            foreach (UltraGridRow row in this.ultraGrid1.Rows)
            {
                if (row.IsGroupByRow && 4 < row.Index && row.Index < 10)
                {
                    row.Expanded = true;
                }
            }

            this.expandedRows.Clear();
            this.expandedRows = this.GetExpandedRows(this.ultraGrid1);
        }

        private DataSet GetData()
        {
            DataSet ds = new DataSet();
            DataTable t1 = ds.Tables.Add("table1");
            t1.Columns.Add("id", typeof(int)).AutoIncrement = true;
            t1.Columns.Add("Name", typeof(string));
            t1.Columns.Add("Quantity", typeof(int));
            t1.Columns.Add("Price", typeof(double));
            t1.Columns.Add("Date", typeof(DateTime));

            ds = this.AddData(ds);

            return ds;
        }

        private DataSet AddData(DataSet ds)
        {
            ds.Tables["table1"].Rows.Add(new object[] { null, "Beverage", 50, 10.2, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Appliances", 5, 90, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "FastFood", 20, 32, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Furniture", 12, 79, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Meat", 40, 10.2, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Fruits", 25, 12, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Vegetables", 30, 13, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Cars", 13, 200, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Tools", 60, 15.3, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Estate", 5, 3000, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Motorcycles", 10, 320, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Clothes", 40, 69, new DateTime(2009, 02, 28) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Shoes", 24, 40.6, new DateTime(2008, 04, 22) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Games", 53, 50, new DateTime(2012, 05, 10) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Books", 42, 22, new DateTime(2005, 10, 05) });
            ds.Tables["table1"].Rows.Add(new object[] { null, "Movies", 38, 19, new DateTime(2009, 02, 28) });

            return ds;
        }

        private ICollection<UltraGridRow> GetExpandedRows(UltraGrid grid)
        {
            ICollection<UltraGridRow> rows = new HashSet<UltraGridRow>(); 
            foreach (UltraGridRow row in grid.Rows)
            {
                if (row.IsGroupByRow)
                {
                    if (row.IsExpanded)
                    {
                        rows.Add(row);
                    }
                }
            }

            return rows;
        }

        private void ScrollToFirstExpandedRow(UltraGrid grid)
        {
            this.ultraGrid1.Rows.CollapseAll(true);
            foreach (UltraGridRow row in grid.Rows)
            {
                if (this.expandedRows.Contains(row))
                {
                    row.Expanded = true;
                    if (this.firstExpandedRow == null)
                    {
                        this.firstExpandedRow = row;
                    }
                }
            }

            grid.ActiveRowScrollRegion.FirstRow = this.firstExpandedRow;
            grid.Selected.Rows.Clear();
            grid.Selected.Rows.Add(this.firstExpandedRow);
            grid.ActiveRow = this.firstExpandedRow;
            this.firstExpandedRow = null;
        }

        private void ButtonScroll_Click(object sender, EventArgs e)
        {
            this.expandedRows.Clear();
            this.expandedRows = this.GetExpandedRows(this.ultraGrid1);
            this.ScrollToFirstExpandedRow(this.ultraGrid1);
        }
    }
}
